@@ -193,7 +193,7 @@ compctlread(char *name, char **args, Options ops, char *reply)
 
     /* only allowed to be called for completion */
     if (!incompctlfunc) {
-	zwarnnam(name, "option valid only in functions called for completion");
+	zwarnnam(name, "option valid only in functions called via compctl");
 	return 1;
     }
 
@@ -3391,7 +3391,7 @@ makecomplistflags(Compctl cc, char *s, int incmd, int compadd)
 		    mflags |= CMF_FILE;
 		    for (n = firstnode(l); n; incnode(n))
 			addmatch(getdata(n), NULL);
-		    mflags &= !CMF_FILE;
+		    mflags &= ~CMF_FILE;
 		}
 		opts[NULLGLOB] = ng;
 	    } else {
